/*
	bd123.c		7/09/87

	% The 123 border.

	OWL 1.1
	Copyright (c) 1986, 1987, by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 7/27/88 jmd 	Upgraded to new border stuff
	 9/12/88 jmd	Added in and out data to objects
	10/05/88 jmd	Prompts now use a fixed (allocated) data buffer
	11/02/88 ted	Changed to new non-window border scheme
	11/20/88 jmd	Added ID to obj struct
	12/20/88 jmd	removed SETSIZE msg

	 5/28/89 jmd	added Shadow support
*/

#include "oakhead.h"
#include "disppriv.h"
#include "bordobj.h"
#include "bordod.h"

/* border object data */

typedef struct {
	border_od	bdd;		  				/* common object super class */
	char 		prompt[BD_PROMPTLEN + 1];	/* space for prompt */
} bd123_od;

int bd_123(objdata, msg, indata, outdata)
	VOID *objdata;			/* object instance data pointer */
	int msg;				/* message */
	VOID *indata;			/* message input data */
	VOID *outdata;			/* message output data */
/*
	This is the 123 border used by the slug menuing system.
	It is actully not a border but simply a function that 
	will intercept requests to display prompt strings and place 
	them beneath the sed.

	The prompt is limited to BD_PROMPTLEN (80) characters in length.
*/
{
	bd123_od 	*bd123d;
	int			 row, len;
	char		*p;
	byte		 attr;

	bd123d = (bd123_od *)objdata;
	
	switch(msg) {
	case OBJM_GETDATASIZE:
		((ogds_struct *) outdata)->odsize = sizeof(bd123_od);
		((ogds_struct *) outdata)->xdsize = sizeof(border_xd);
		((ogds_struct *) outdata)->id = ID_BD123;
		break;

	case OBJM_OPEN:
		if (!border_DoRaw(&bd123d->bdd, msg, indata, outdata)) {
			return(FALSE);
		}
		bd123d->prompt[0] = '\0';
		bord_SetSides(bd123d->bdd.win, 0, 0, 1, 0);
		break;

	case BDM_PROMPT:	/* set the border prompt string */
		p = (indata != NULL) ? ((char *) indata) : "";
		strncpy(bd123d->prompt, p, BD_PROMPTLEN);
		bd123d->prompt[BD_PROMPTLEN] = '\0';

		/* update the prompt (paint bottom border row) */
		row = bord_GetHeight(bd123d->bdd.win) - 1;
		len = bord_GetWidth(bd123d->bdd.win);
		win_PaintRow(bd123d->bdd.win, row, 0, len);
		break;

	case BDM_SHADOW:
	case BDM_DRAW:		/* draw the message */
		row = bord_GetHeight(bd123d->bdd.win) - 1;
		len = bord_GetWidth(bd123d->bdd.win);
		attr = (msg == BDM_DRAW) ? 
			bord_GetAttr(bd123d->bdd.win) : win_GetShadowAttr(bd123d->bdd.win);

		ptd_DrawString((ptd_struct *)indata, row, 0, bd123d->prompt, attr, len);

		/* else no break; pass message up to superclass */

	default:
		return(border_DoRaw(&bd123d->bdd, msg, indata, outdata));
	}
	return(TRUE);
}

