/*
	oakpriv.h

	% Compiler and OS dependent Internal declarations used by Oakland C-tools.

	OWL 1.1
	Copyright (c) 1986, 1987, 1988, by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	12/23/88 jmd	Added OS2 define
	 1/16/89 jmd	Removed limits.h from apollo case
	 4/09/89 ted	Removed OS2 defines. Put them in our makefile instead.
	 8/07/89 jmd	added omemmove to UNIX. move MetaWare define from oakland.h
	 8/18/89 ted	added owl_InitSysWins macro.
	 8/22/89 ted	added USHRT_MAX macro.
	 9/11/89 jmd	added BORDER_CHARS macro.

	This file depends on a symbol for the Compiler or Operating System
	being #define'd.

	Use one of the following:

	M5
	TC
	LC
	OAK_UNIX
	OAK_APOLLO
	OAK_SUN
	OAK_VMS
	OAK_STRATUS
	OAK_DG

	Note:  the BORDER_CHARS macro turns on fancy characters used by 
		   certain borders (such as bd_std).	
*/
/* -------------------------------------------------------------------------- */
#ifdef M5	/*	  	Microsoft C version 5.x Dependent Code 	*/
#	include <limits.h>
	int _CDECL inp(unsigned int);
	int _CDECL outp(unsigned int, int);

#	define DIGPRIV	near
#	define OWLPRIV	near
#	define CSPRIV	near

#	define BORDER_CHARS
#endif

#ifdef TC	/*	  	Turbo C Dependent Code 		*/
#	include <limits.h>

#	define DIGPRIV	near
#	define OWLPRIV	near
#	define CSPRIV	near

#	define BORDER_CHARS
#endif

#ifdef MW	/* MetaWare High C stuff */
#	include <limits.h>
#	define DIGPRIV
#	define OWLPRIV
#	define CSPRIV

	extern VOID *omemmove(_arg3(VOID *, VOID *, SIZE_T));
#	define memmove(dest,src,cnt)	omemmove(dest, src, cnt)

#	define BORDER_CHARS
#endif

#ifdef LC	/*	  	Lattice C Dependent Code	*/
#	include <limits.h>
#	define memmove(dest, src, cnt)		memcpy(dest, src, cnt)

#	define DIGPRIV	near
#	define OWLPRIV	near
#	define CSPRIV	near
#	define BORDER_CHARS
#endif

#ifdef W7	/*	  	Watcom 7.x Dependent Code 	*/
#	include <limits.h>
#	define DIGPRIV	near
#	define OWLPRIV	near
#	define CSPRIV	near
#	define BORDER_CHARS
#endif

#ifdef OAK_UNIX		/*	  	UNIX Dependent Code */
/* from limits.h */
#	define UINT_MAX		((unsigned int) ~0)		/* max value for unsigned int */
#	define USHRT_MAX	((unsigned short) ~0)	/* max value for unsigned short */
#	define ULONG_MAX	((unsigned long) ~(0L))	/* max value for unsigned long */

/* convert memmove to omemmove */

	extern VOID *omemmove(_arg3(VOID *, VOID *, SIZE_T));
#	define memmove(dest,src,cnt)	omemmove(dest, src, cnt)

#	define DIGPRIV
#	define OWLPRIV
#	define CSPRIV
#endif

#ifdef OAK_APOLLO
/* from limits.h */
#	define UINT_MAX		((unsigned int) ~0)		/* max value for unsigned int */
#	define USHRT_MAX	((unsigned short) ~0)	/* max value for unsigned short */
#	define ULONG_MAX	((unsigned long) ~(0L))	/* max value for unsigned long */
#	define memmove(dest, src, cnt)		memcpy(dest, src, cnt)

#	define DIGPRIV
#	define OWLPRIV
#	define CSPRIV

#	define owl_InitSysWins()	gpr_InitSysWins()
	void gpr_InitSysWins(_arg1(void));
#endif

#ifdef OAK_SUN		/*	  	SUN Dependent Code */
/* from limits.h */
#	define UINT_MAX		((unsigned int) ~0)		/* max value for unsigned int */
#	define USHRT_MAX	((unsigned short) ~0)	/* max value for unsigned short */
#	define ULONG_MAX	((unsigned long) ~(0L))	/* max value for unsigned long */

/* convert memmove to memcpy for ANSI compatibility */
#	define memmove(dest, src, cnt)		memcpy(dest, src, cnt)

#	define DIGPRIV
#	define OWLPRIV
#	define CSPRIV
#endif

#ifdef OAK_VMS		/*	  	VMS Dependent Code */
/* from limits.h */
#	define UINT_MAX		((unsigned int) ~0)		/* max value for unsigned int */
#	define USHRT_MAX	((unsigned short) ~0)	/* max value for unsigned short */
#	define ULONG_MAX	((unsigned long) ~(0L))	/* max value for unsigned long */

/* convert memmove to omemmove */
	extern VOID *omemmove(_arg3(VOID *, VOID *, SIZE_T));
#	define memmove(dest,src,cnt)	omemmove(dest, src, cnt)

#	define DIGPRIV
#	define OWLPRIV
#	define CSPRIV
#endif

#ifdef OAK_STRATUS		/*	  	STRATUS Dependent Code */
#	include <limits.h>

#	define DIGPRIV
#	define OWLPRIV
#	define CSPRIV
#endif

#ifdef OAK_DG	/*	  	Data General C Dependent Code	*/
/* from limits.h */
#	define UINT_MAX		0x7FFFFFFF		/* max value for unsigned int 		*/
#	define USHRT_MAX	0xFFFF			/* max value for unsigned short		*/
#	define ULONG_MAX	0x7FFFFFFF		/* max value for unsigned long		*/
#	define memmove(dest, src, cnt)		memcpy(dest, src, cnt)
#	define OSTACK_MINUS			/* direction of stack growth */

#	define DIGPRIV
#	define OWLPRIV
#	define CSPRIV
#endif

#ifndef owl_InitSysWins
#	define owl_InitSysWins()
#endif
/* -------------------------------------------------------------------------- */

